/* ===================================================[ PlaylistInterface.hh ]=
 * PlaylistInterface.hh - interface with the AlsaPlayer playlist
 * Copyright (C) 2002 Hubert Chan <hubert@uhoreg.ca>
 *
 * ----------------------------------------------------------------( License )-
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * ------------------------------------------------------------( Description )-
 *
 * This file fetches, keeps track of, and modifies the AlsaPlayer playlist.
 *
 * ------------------------------------------------------------------------- */

#include <string>
#include <sstream>

#include "PlaylistInterface.hh"

inline std::string make_string(PlayItem &item)
{
  if (item.Parsed())
  {
    return std::string(item.title) + " - " + item.artist;
  }
  else
  {
    return item.filename;
  }
}

LIRC_PlaylistInterface::LIRC_PlaylistInterface(SessionData &_sessiondata)
  : sessiondata(_sessiondata),
    current(-1),
    selected(-1),
    display_top(0)
{
  sessiondata.display->print(MODE_PLAYLIST, 0, "<empty>");
  sessiondata.playlist->Register(this);
}

LIRC_PlaylistInterface::~LIRC_PlaylistInterface()
{
  sessiondata.playlist->UnRegister(this);
}

void LIRC_PlaylistInterface::update_display(unsigned int top,
					    unsigned int bottom)
{
  bottom = min(bottom, sessiondata.display->getHeight());
  for (unsigned int i = top + display_top;
       i < playlist.size() && top < bottom;
       i++, top++)
  {
    sessiondata.display->print(MODE_PLAYLIST, top,
			       "  " + make_string(playlist[i]));
  }

  // fill the rest with blank lines
  for (; top < bottom; top++)
    sessiondata.display->print(MODE_PLAYLIST, top, "");
}

void LIRC_PlaylistInterface::CbSetCurrent(unsigned pos)
{
  // erase old P indicator
  if (display_top <= current &&
      current < display_top + int(sessiondata.display->getHeight()))
  {
    sessiondata.display->print(MODE_PLAYLIST, current - display_top,
			       "  " + make_string(playlist[current]));
  }

  current = pos - 1;

  if (current < 0)
    sessiondata.display->print(MODE_PLAYER, 1, "");
  else
  {
    // update the current song display on the main screen
    sessiondata.display->print(MODE_PLAYER, 1, make_string(playlist[current]));

    // update the playlist display
    if (selected < 0)
    { // if no song is selected, the display follows the current song
      // calculate the display position of the new current song
      int disp_pos = current - display_top;

      if (disp_pos < 0)
      { // song comes before current display -- put at the bottom of the
	// display
	display_top = max(0, current - int(sessiondata.display->getHeight()) + 1);
	update_display();
      }
      else if (unsigned(disp_pos) >= sessiondata.display->getHeight())
      { // song comes after current display -- put at the top of the display
	display_top = current;
	update_display();
      }
    }

    // add new P indicator
    if (display_top <= current &&
	current < display_top + int(sessiondata.display->getHeight()))
    {
      sessiondata.display->print(MODE_PLAYLIST, pos - display_top - 1,
				 "P " + make_string(playlist[current]));
    }
  }
}

void LIRC_PlaylistInterface::CbInsert(std::vector<PlayItem> &items, unsigned pos)
{
  playlist.insert(playlist.begin()+pos, items.begin(), items.end());

  if (unsigned(display_top) + 1 < pos + items.size())
    update_display(int(pos) - display_top);
}

void LIRC_PlaylistInterface::CbUpdated(PlayItem &item, unsigned pos)
{
  playlist[pos-1] = item;

  if (pos == (unsigned) (current + 1))
    sessiondata.display->print(MODE_PLAYER, 1, make_string(playlist[current]));

  if (unsigned(display_top) < pos && pos <= display_top + sessiondata.display->getHeight())
    sessiondata.display->print(MODE_PLAYLIST, pos - display_top - 1,
			       "  " + make_string(item));
}

void LIRC_PlaylistInterface::CbRemove(unsigned start, unsigned end)
{
  playlist.erase(playlist.begin()+start, playlist.begin()+end);
  update_display(int(start) - display_top - 1);
}

void LIRC_PlaylistInterface::CbClear()
{
  playlist.clear();
  current = -1;
  selected = -1;
  sessiondata.display->print(MODE_PLAYLIST, 0, "<empty>");
  for (unsigned i = 1; i < sessiondata.display->getHeight(); i++)
    sessiondata.display->print(MODE_PLAYLIST, i, "");
}
