/* =============================================================[ ap-lirc.cc ]=
 * ap-lirc.cc - LIRC interface plugin main file
 * Control AlsaPlayer through LIRC.
 * Copyright (C) 2002 Hubert Chan <hubert@uhoreg.ca>
 *
 * ----------------------------------------------------------------( License )-
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * ------------------------------------------------------------( Description )-
 *
 * This program allows one to control AlsaPlayer using a remote control through
 * LIRC (Linux Infrared Remote Control).
 *
 * Lots of code ripped from irexec.c and the AlsaPlayer text and gtk interface
 * plugins.  Commands based on lirc-xmms-plugin.
 *
 * ------------------------------------------------------------------------- */

#include <iostream>
#include <sstream>
#include <iomanip>
//#include <ios>

#include <alsaplayer/AlsaPlayer.h>
#include <alsaplayer/SampleBuffer.h>
#include <alsaplayer/CorePlayer.h>
#include <alsaplayer/Playlist.h>
#include <alsaplayer/alsaplayer_error.h>
#include <alsaplayer/interface_plugin.h>
#include <alsaplayer/prefs.h>

#include <lirc/lirc_client.h>

#include "Display.hh"
#include "SessionData.hh"
#include "CommandSet.hh"
#include "PlaylistInterface.hh"

namespace {

  //------------------------------------------------------( Global variables )-
  /* note: global variables are evil.  I don't see any better way to do things,
   * though.  */

  struct lirc_config *config;


  template <class T>
  inline std::string slider_string(T min, T max, T value, int size)
  {
    std::string str(size, '-');
    int pos = int((value - min) * size / (max - min));
    if (pos < 0)
      pos = 0;
    else if (pos >= size)
      pos = size-1;
    str[pos] = '|';
    return str;
  }

  //-------------------------------------------------------------( Notifiers )-

  void volume_changed(void *data, int volume)
  {
    SessionData *sd = static_cast<SessionData*>(data);
    std::ostringstream str;
    str << "vol: ["
	<< slider_string(0, 100, volume, sd->display->getWidth() - 11)
	<< "] " << std::setw(3) << volume;
    sd->display->print(MODE_PLAYER, 2, str.str());
  }

  void speed_changed(void *data, float new_speed)
  {
    SessionData *sd = static_cast<SessionData*>(data);
    std::ostringstream str;
    str.setf(ios::fixed, ios::floatfield); // FIXME: make this standards compliant
    str << "speed: ["
	<< slider_string(-2.0, 2.0, double(new_speed), sd->display->getWidth() - 15)
	<< "] " << std::setprecision(2) << std::setw(5) << new_speed;
    sd->display->print(MODE_PLAYER, 2, str.str());
  }

  void pan_changed(void *data, int new_pan)
  {
    SessionData *sd = static_cast<SessionData*>(data);
    std::ostringstream str;
    str << "pan: ["
	<< slider_string(-100, 100, new_pan, sd->display->getWidth() - 11)
	<< "] " << std::setw(4) << new_pan;
    sd->display->print(MODE_PLAYER, 2, str.str());
  }

  void position_notify(void *data, int frame)
  {
     SessionData *sd = static_cast<SessionData*>(data);
     int num_frames = sd->coreplayer->GetFrames();
     if (num_frames <= 0)
       return;
     int current_time = sd->coreplayer->GetCurrentTime() / 100;

     std::ostringstream str;
     str << '['
	 << slider_string(0, num_frames, frame, sd->display->getWidth() - 8)
	 << "] " << std::setw(2) << (current_time/60) << ':'
	 << std::setfill('0') << std::setw(2) << (current_time%60);
     sd->display->print(MODE_PLAYER, 3, str.str());
  }

  void start_notify(void *data)
  {
  }

  void stop_notify(void *data)
  {
  }


  //------------------------------------------------------( Plugin functions )-

  int interface_lirc_init()
  {
#ifdef DEBUG
    std::cout << "In lirc_init\n";
#endif

    if(lirc_init("alsaplayer",1)==-1)
    {
      std::cerr << "lirc interface error: could not initialize lirc\n";
      return 1; //?
    }

    if(lirc_readconfig(NULL,&config,NULL)!=0)
    {
      std::cerr << "lirc interface error: could not read configuration\n";
      lirc_deinit();
      return 1; //?
    }

    CommandSet::init();

    return 1;
  }

  int interface_lirc_running()
  {
    return 1;
  }

  int interface_lirc_stop()
  {
    return 1;
  }

  void interface_lirc_close()
  {
#ifdef DEBUG
    std::cout << "In lirc_close\n";
#endif

    lirc_freeconfig(config);
    lirc_deinit();
    return;
  }

  int interface_lirc_start(Playlist *playlist, int argc, char **argv)
  {
#ifdef DEBUG
    std::cout << "In lirc_start\n";
#endif

    SessionData sessiondata(playlist);

    LIRC_PlaylistInterface playlist_interface(sessiondata);

    coreplayer_notifier notifier;

    memset(&notifier, 0, sizeof(notifier));
    notifier.volume_changed = volume_changed;
    notifier.speed_changed = speed_changed;
    notifier.pan_changed = pan_changed;
    notifier.position_notify = position_notify;
    notifier.start_notify = start_notify;
    notifier.stop_notify = stop_notify;

    playlist->RegisterNotifier(&notifier, &sessiondata);

    char *code;
    char *c;
    char *args;
    int ret;
    CommandSet callbackset(sessiondata);

    while(lirc_nextcode(&code)==0)
    {
      if(code==NULL) continue;
      while((ret=lirc_code2char(config,code,&c))==0 &&
	    c!=NULL)
      {
	int i;
	for (i = 0; c[i] != '\0'; i++)
	  c[i] = tolower(c[i]);
#ifdef DEBUG
	std::cout << "command:" << c << endl;
#endif

	c = strdup(c);
	/* separate command from arguments -- split at the first space */
	if ((args = strchr(c, ' ')) != NULL)
	{
	  *args = '\0';
	  args++;
	  while (*args == ' ')
	    args++;
	}

	if (strcmp(c, "quit") == 0)
	{
	  free(code);
	  playlist->UnRegisterNotifier(&notifier);
	  return 0;
	}
	callbackset.call(c,args ? args : "");

	free(c);
      }

      free(code);
      if(ret==-1) break;
    }

    playlist->UnRegisterNotifier(&notifier);

#ifdef DEBUG
    std::cout << "Exiting lirc_start\n";
#endif

    return 0;
  }

  interface_plugin default_plugin =
  {
    INTERFACE_PLUGIN_VERSION,
    "LIRC interface v0.4",
    "Hubert Chan",
    NULL,
    interface_lirc_init,
    interface_lirc_start,
    interface_lirc_running,
    interface_lirc_stop,
    interface_lirc_close
  };

}

extern "C" {

interface_plugin *interface_plugin_info()
{
  return &default_plugin;
}

}

// end of ap-lirc.cc
